/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Calendar;
import java.util.LinkedHashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.DataState;

/**
 * Enthaelt nur Konstanten, Hilfswerte und einfache statische Hilfsmethoden.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 */
public final class AbfrPufferKonstanten {

	private AbfrPufferKonstanten() {
		// es gibt keine Instanzen der Klasse
	}

	/* TYPEN */
	public static final String TYP_ARCHIV = "typ.archiv";
	public static final String TYP_KONFIGURATIONSBEREICH = "typ.konfigurationsBereich";
	public static final String TYP_STEUERMODUL = "typ.steuerModul";
	public static final String TYP_ACHSLAST_MESSSTELLE = "typ.achsLastMessStelle";
	public static final String TYP_ABFRAGEMODUL = "typ.abfrageModul";

	/* DATENENDGERAET TYPEN */
	public static final String TYP_DE_AXL = "typ.deAxl";
	public static final String TYP_DE_LVE = "typ.deLve";
	public static final String TYP_DE_SYS = "typ.deSys";

	/* ATTRIBUTGRUPPEN */
	public static final String ATG_ARCHIV = "atg.archiv";
	public static final String ATG_DE = "atg.de";
	public static final String ATG_FAHRSTREIFEN = "atg.fahrStreifen";
	public static final String ATG_ACHSLASTMESSSTELLE = "atg.achsLastMessStelle";
	public static final String ATG_ABFRAGEMODUL = "atg.abfrageModul";

	/* TLS ATRIBUTGRUPPEN */
	public static final String ATG_TLS_GLODEFEHLER = "atg.tlsGloDeFehler";

	/* TLS AXL ATRIBUTGRUPPEN */
	public static final String ATG_TLS_AXLABRUF = "atg.tlsAxlAbrufPufferInhalt";
	public static final String ATG_TLS_AXLEINZELERGEBNIS_MELDUNG = "atg.tlsAxlEinzelErgebnisMeldungAchsLastenUndKfzGesamtGewichteVersion1Bis2";
	public static final String ATG_TLS_AXLERGEBNIS_MELDUNG = "atg.tlsAxlErgebnisMeldungAchsLastDatenVersion10";
	public static final String ATG_TLS_AXLBETRIEBSPARAM = "atg.tlsAxlBetriebsParameter";

	public static final String ATG_TLS_AXLNEGATIVE_QUITTUNG = "atg.tlsAxlNegativeQuittung";

	/* TLS LVE ATRIBUTGRUPPEN */
	public static final String ATG_TLS_LVEABRUF = "atg.tlsLveAbrufPufferInhalt";
	public static final String ATG_TLS_LVEBETRIEBSPARAM = "atg.tlsLveBetriebsParameter";
	public static final String ATG_TLS_LVENEGATIVE_QUITTUNG = "atg.tlsLveNegativeQuittung";

	/* ASPEKTE */
	public static final String ASP_EIGENSCHAFTEN = "asp.eigenschaften";
	public static final String ASP_PARAMETERIST = "asp.parameterIst";
	public static final String ASP_PARAMETERSOLL = "asp.parameterSoll";
	public static final String ASP_TLS_ANTWORT = "asp.tlsAntwort";
	public static final String ASP_TLS_ABRUF = "asp.tlsAbruf";
	public static final String ASP_PARAM_VORGABE = "asp.parameterVorgabe";

	/* ATRIBUTE */
	public static final String ATR_DATENSPZIFIKATION = "DatenSpezifikation";
	public static final String ATR_FAHRSTREIFENQUELLE = "FahrStreifenQuelle";
	public static final String ATR_AXLMESSSTELLENQUELLE = "AchsLastMessStellenQuelle";

	public static final String ATR_OBJEKT = "Objekt";
	public static final String ATR_PARAMETERSATZ = "ParameterSatz";

	public static final String ATR_INTERVALLDAUER_KZD = "IntervallDauerKurzZeitDaten";
	public static final String ATR_VERSION_KZD = "VersionKurzZeitDaten";

	public static final String ATR_INTERVALLDAUER_LZ = "IntervallDauerLangZeit";
	public static final String ATR_VERSION_LZD = "VersionLangZeitDaten";

	public static final String ATR_ALPHA1 = "alpha1";
	public static final String ATR_ALPHA2 = "alpha2";
	public static final String ATR_LANGE_GW = "LängenGrenzWert";
	public static final String ATR_ART_MWB = "ArtMittelWertBildung";
	public static final String ATR_START_MWB = "StartMittelWertBildung";
	public static final String ATR_INTERVALL_AXL = "Erfassungsintervalldauer";

	public static final String ATR_ANZAHL = "Anzahl";
	public static final String ATR_ARCHIVIEREN = "Archivieren";
	public static final String ATR_DE = "De";
	public static final String ATR_DE_KANAL = "DEKanal";
	public static final String ATR_DE_FEHLERSTATUS = "DEFehlerStatus";
	public static final String ATR_DE_KANALSTATUS = "DEKanalStatus";
	public static final String ATR_DE_PROJEKTSTATUS = "DEProjektierungsStatus";
	public static final String ATR_HERSTELLERDEFCODE = "HerstellerDefinierterCode";
	public static final String ATR_HERSTELLER = "Hersteller";
	public static final String ATR_EAK = "Eak";
	public static final String ATR_CODE = "Code";
	public static final String ATR_EINZELLTELEGRAMM = "EinzelTelegramm";

	public static final String ATR_LVEABFRAGEZYKLUS = "LveAbfrageZyklus";
	public static final String ATR_LVEABFRAGEOFFSET = "LveAbfrageOffset";
	public static final String ATR_AXLABFRAGEZYKLUS = "AxlAbfrageZyklus";
	public static final String ATR_AXLABFRAGEOFFSET = "AxlAbfrageOffset";

	public static final String ZEITPUNKT = "Zeitpunkt";

	/**
	 * Eine Abbildung von DatenStatus Fehlercoden zur Fehlerbeschreibungen.
	 */
	public static final Map<DataState, String> DATEN_STATUS = new LinkedHashMap<>();

	static {
		AbfrPufferKonstanten.DATEN_STATUS.put(DataState.NO_DATA, "keine Daten");
		AbfrPufferKonstanten.DATEN_STATUS.put(DataState.NO_SOURCE, "keine Daten (keine Quelle)");
		AbfrPufferKonstanten.DATEN_STATUS.put(DataState.NO_RIGHTS, "keine Daten (keine Rechte)");
		AbfrPufferKonstanten.DATEN_STATUS.put(DataState.POSSIBLE_GAP, "keine Daten (potentielle Datenlücke)");
		AbfrPufferKonstanten.DATEN_STATUS.put(DataState.UNAVAILABLE_BLOCK, "Ausgelagerter Bereich");
		AbfrPufferKonstanten.DATEN_STATUS.put(DataState.DELETED_BLOCK, "Gelöschter Bereich");
		AbfrPufferKonstanten.DATEN_STATUS.put(DataState.END_OF_ARCHIVE, "Ende des Archivanfragezeitraums");
		AbfrPufferKonstanten.DATEN_STATUS.put(DataState.INVALID_SUBSCRIPTION, "keine Daten (fehlerhafte Anmeldung)");
	}

	/**
	 * Eine Stunde in ms.
	 */
	public static final long STUNDE_IN_MS = 60L * 60L * 1000L;
	/**
	 * Eine Minute in ms.
	 */
	public static final long MINUTE_IN_MS = 60L * 1000L;

	/**
	 * Maximaler Anzahl der Abgefragten Datensaetzen in einer Abfrage (255).
	 */
	public static final long MAX_DATENSAETZE = 255;

	/**
	 * Maximaler Intervall von LangzeitdatenErfassung.
	 */
	public static final long INTERVALL_MAX = 24L * AbfrPufferKonstanten.STUNDE_IN_MS;
	/**
	 * Beginnzeitpunkt des Moduls, Defaultzeitpunkt wenn es keinen bessern Wert.
	 * gibt
	 */
	public static final long ZEIT_BEGINN;

	static {
		final Calendar cal = Calendar.getInstance();
		cal.clear();
		cal.set(2009, 6, 1, 0, 0, 0); // ACHTUNG Januar=0 !
		ZEIT_BEGINN = cal.getTimeInMillis();
	}

	/**
	 * Basis Timeout Zeitintervalle aus der Feinspezifikation.
	 */
	static final long BASIS_TIMEOUT_LVE = 500;
	static final long BASIS_TIMEOUT_AXL = 500;

	/**
	 * Zeitkonstante die Bestimmt den Intervall, in dem weniger als 10 Status
	 * Aenderungen bestehen sollen.
	 *
	 * Sonst werden wir die Station als fehlerhaft bezeichnen.
	 */
	static final long STATUS_AENDERUNGEN_MINIMUM_DELTA = AbfrPufferKonstanten.STUNDE_IN_MS / 2;

	/**
	 * Konveriert Zeitcode aus dem LveBetriebsParameter Telegramm zur Zeit in
	 * ms.
	 *
	 * @param zeitCode
	 *            Zeitcode
	 * @return Zeit in ms
	 */
	public static long convertLveZeitCodeZurZeit(final long zeitCode) {
		return (zeitCode - 128) * AbfrPufferKonstanten.STUNDE_IN_MS;
	}

	/**
	 * Konveriert Zeit in ms zum Zeitcode in dem LveBetriebsParameter Telegramm.
	 *
	 * @param zeit
	 *            Zeit in ms
	 * @return Zeitcode
	 */
	public static long convertZeitZurLveZeitCode(final long zeit) {
		return (zeit / AbfrPufferKonstanten.STUNDE_IN_MS) + 128;
	}

	/**
	 * Konveriert Zeitcode aus dem AxlBetriebsParameter Telegramm zur Zeit in
	 * ms.
	 *
	 * @param zeitCode
	 *            Zeitcode
	 * @return Zeit in ms
	 */
	public static long convertAxlZeitCodeZurZeit(final long zeitCode) {
		return zeitCode * (AbfrPufferKonstanten.STUNDE_IN_MS / 4);
	}

	/**
	 * Konveriert Zeit in ms zum Zeitcode in dem AxlBetriebsParameter Telegramm.
	 *
	 * @param zeit
	 *            Zeit in ms
	 * @return Zeitcode
	 */
	public static long convertZeitZurAxlZeitCode(final long zeit) {
		return zeit / (AbfrPufferKonstanten.STUNDE_IN_MS / 4);
	}

	/**
	 * Konvertiert StatsCode zum boolean.
	 *
	 * @param status
	 *            Statuscode
	 * @return Boolean ( true = StatusOK )
	 */
	public static boolean convertStatusZumBoolean(final int status) {
		if (status == 0) {
			return true;
		}
		return false;
	}
}
